/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.filetransfer.service;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;

import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.SourceHelper;

/**
 * The file transfer BC JBI listener. Writes a file to hard disk on incoming JBI
 * message.
 * 
 * @version
 * @since Petals 2.0
 * @author Adrien LOUIS - EBMWebSourcing
 * 
 */

public class GetFileService extends AbstractGetOperationService {

    public GetFileService(ConfigurationExtensions extensions, Logger logger) {
        super(extensions, logger);
    }

    /**
     * Process the exchange : get the file in the configured directory, <br>
     * using filepattern set in the exchange <br>
     * file is returned as OUT response (assume the file is an XML file) <br>
     * return the first file <br>
     * the file is move/deleted after read
     */
    @Override
    public void process(Exchange exchange) {
        logger.log(Level.INFO, "JBI Exchange received - getFile service requested");
        if (exchange.isInOnlyPattern() || exchange.isRobustInOnlyPattern()) {
            exchange
                    .setError(new JBIException(
                            "the exchange pattern is not compatible. Operation getFile is InOut or InOptionalOut."));
        } else {
            try {
                String filePattern = extractFilePattern(exchange);
                File readDirectory = FileTransferUtils.getReadDirectory(extensions);
                logger.log(Level.INFO, "read files from : " + readDirectory.getAbsolutePath());
                logger.log(Level.INFO, "file pattern : " + filePattern);
                File[] files = listFiles(readDirectory, filePattern);
                if (files.length > 0) {
                    File file = files[0];
                    // check if the file can be read
                    if (!FileTransferUtils.checkAccess(file)) {
                        throw new IOException("File " + file.getName() + " is locked.");
                    }

                    logger.log(Level.INFO, "file found : " + file.getName());
                    // move file in a work or backup dir before processing
                    File processedFile = FileTransferUtils.moveFile(file, extensions);
                    // copy file
                    NormalizedMessage out = exchange.getOutMessage();
                    FileTransferUtils.copyFileToContent(processedFile, out);
                } else {
                    try {
                        Fault fault = exchange.createFault();
                        fault.setContent(SourceHelper
                                .createSource("Unable to find any file for the pattern "
                                        + filePattern));
                        exchange.setFault(fault);
                    } catch (PEtALSCDKException e) {
                        logger.log(Level.SEVERE, e.getMessage());
                        exchange.setError(new JBIException(e));
                    }
                }
                logger.log(Level.INFO, "JBI Exchange received - getFile service requested");
            } catch (MessagingException e) {
                logger.log(Level.SEVERE, e.getMessage(), e);
                exchange.setError(e);
            } catch (IOException e) {
                try {
                    logger.warning("can not retrieve file(s). " + e.getClass().getName() + " : "
                            + e.getMessage());
                    setIOFaultOnExchange(exchange, e.getMessage());
                } catch (JBIException e1) {
                    logger.log(Level.SEVERE, e1.getMessage(), e1);
                    exchange.setError(e1);
                }
            }
            logger.log(Level.INFO, "getFile service is correctly terminated");
        }
    }
}
