/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.filetransfer.service;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;

import org.ow2.petals.bc.filetransfer.FileTransferConstants;
import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.message.Exchange;

/**
 * The file transfer BC JBI listener. Writes a file to hard disk on incoming JBI
 * message.
 * 
 * @version
 * @since Petals 2.0
 * @author Adrien LOUIS - EBMWebSourcing
 * 
 */

public class GetFilesService extends AbstractGetOperationService {

    public GetFilesService(ConfigurationExtensions extensions, Logger logger) {
        super(extensions, logger);
    }

    /**
     * Process the exchange : get the files in the configured directory, <br>
     * using filepattern set in the exchange <br>
     * files are returned as attachment in the OUT response <br>
     * OUT message contains files names <br>
     * the file is move/deleted after read
     */
    @Override
    public void process(Exchange exchange) {
        logger.log(Level.INFO, "JBI Exchange received - getFiles service requested");
        if (exchange.isInOnlyPattern() || exchange.isRobustInOnlyPattern()) {
            exchange
                    .setError(new JBIException(
                            "the exchange pattern is not compatible. Operation getFiles is InOut or InOptionalOut."));
        } else {

            try {
                String filePattern = extractFilePattern(exchange);
                File readDirectory = FileTransferUtils.getReadDirectory(extensions);
                logger.log(Level.INFO, "read files from : " + readDirectory.getAbsolutePath());
                logger.log(Level.INFO, "file pattern : " + filePattern);
                File[] files = listFiles(readDirectory, filePattern);
                NormalizedMessage out = exchange.getOutMessage();
                List<String> fileNameList = new ArrayList<String>();
                if (files.length > 0) {
                    for (File file : files) {
                        logger.info("file found : " + file.getName());
                        // check if file can be read
                        if (!FileTransferUtils.checkAccess(file)) {
                            logger.warning(file.getName() + " is locked, don't process it");
                        }
                        // move file in a work or backup dir before processing
                        File processedFile = FileTransferUtils.moveFile(file, extensions);
                        // copy file as attachment
                        FileTransferUtils.addFileAsAttachment(processedFile, file.getName(), out);
                        fileNameList.add(file.getName());
                    }
                }
                // generate OUT response report
                String report = generateFileNameList(fileNameList);
                exchange.setOutMessageContent(report);
            } catch (MessagingException e) {
                exchange.setError(e);
                logger.log(Level.SEVERE, e.getMessage(), e);
            } catch (IOException e) {
                try {
                    logger.warning("can not retrieve file(s). " + e.getClass().getName() + " : "
                            + e.getMessage());
                    setIOFaultOnExchange(exchange, e.getMessage());
                } catch (JBIException e1) {
                    logger.log(Level.SEVERE, e1.getMessage(), e1);
                    exchange.setError(e1);
                }
            }
            logger.log(Level.INFO, "getFiles service is correctly terminated");
        }
    }

    /**
     * generates the XML representation of the file name list
     * 
     * @param names
     * @param namespace
     * @return
     */
    private static String generateFileNameList(List<String> names) {
        StringBuffer result = new StringBuffer();
        result.append("<tns:getFilesResponse xmlns:tns='");
        result.append(FileTransferConstants.GET_FILES_SERVICE_NS);
        result.append("'>");
        for (String string : names) {
            result.append("<tns:filename>");
            result.append(string);
            result.append("</tns:filename>");
        }
        result.append("</tns:getFilesResponse>");
        return result.toString();
    }
}
